////////////////////////////////////////////////////////////////////////
// Fire01.c
// Mochilote - www.cpcmania.com
////////////////////////////////////////////////////////////////////////
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


////////////////////////////////////////////////////////////////////////
//------------------------ Generic functions -------------------------//
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//GetTime()
////////////////////////////////////////////////////////////////////////
unsigned char char3,char4;

unsigned int GetTime()
{
	unsigned int nTime = 0;

	__asm
		CALL #0xBD0D ;KL TIME PLEASE
		PUSH HL
		POP DE
		LD HL, #_char3
		LD (HL), D
		LD HL, #_char4
		LD (HL), E
	__endasm;

	nTime = (char3 << 8) + char4;

	return nTime;
}
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//SetColor
////////////////////////////////////////////////////////////////////////
void SetColor(unsigned char nColorIndex, unsigned char nPaletteIndex)
{
	__asm
		ld a, 4 (ix)
		ld b, 5 (ix)
		ld c, b
		call #0xBC32 ;SCR SET INK
	__endasm;
}
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//SetPalette
////////////////////////////////////////////////////////////////////////
void SetPalette(const unsigned char *pPalette, unsigned char nNumColors)
{
  unsigned char nColor = 0;

  for(nColor = 0; nColor < nNumColors; nColor++)
    SetColor(nColor, pPalette[nColor]);
}
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//SetMode
////////////////////////////////////////////////////////////////////////
void SetMode(unsigned char nMode)
{
	__asm
		ld a, 4 (ix)
		call #0xBC0E ;SCR_SET_MODE
	__endasm;
}
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//SetBorder
////////////////////////////////////////////////////////////////////////
void SetBorder(unsigned char nColor)
{
	__asm
		ld b, 4 (ix)
		ld c, b
		call #0xBC38 ;SCR_SET_BORDER
	__endasm;
}
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//SetCursor
////////////////////////////////////////////////////////////////////////
void SetCursor(unsigned char nColum, unsigned char nLine)
{
	__asm
		ld h, 4 (ix)
		ld l, 5 (ix)
		call #0xBB75 ;TXT SET CURSOR
	__endasm;
}
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//SetMode0PixelColor
////////////////////////////////////////////////////////////////////////
void SetMode0PixelColor(unsigned char *pByteAddress, unsigned char nColor, unsigned char nPixel)
{
	unsigned char nByte = *pByteAddress;

	if(nPixel == 0)
	{
		nByte &= 85;

		if(nColor & 1)
			nByte |= 128;

		if(nColor & 2)
			nByte |= 8;

		if(nColor & 4)
			nByte |= 32;

		if(nColor & 8)
			nByte |= 2;
	}
	else
	{
		nByte &= 170;

		if(nColor & 1)
			nByte |= 64;

		if(nColor & 2)
			nByte |= 4;

		if(nColor & 4)
			nByte |= 16;

		if(nColor & 8)
			nByte |= 1;
	}

	*pByteAddress = nByte;
}
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//PutPixelMode0
////////////////////////////////////////////////////////////////////////
void PutPixelMode0(unsigned char nX, unsigned char nY, unsigned char nColor)
{
	unsigned char nPixel = 0;
	unsigned int nAddress = 0xC000 + ((nY / 8) * 80) + ((nY % 8) * 2048) + (nX / 2);
	nPixel = nX % 2;

	SetMode0PixelColor((unsigned char *)nAddress, nColor, nPixel);
}
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//---------------- Specific variables and functions-------------------//
////////////////////////////////////////////////////////////////////////

#define NUM_COLORS 6
const unsigned char FirePalette[NUM_COLORS] = {0, 3, 6, 15, 24, 26};

#define FIRE_WIDTH 32
#define FIRE_HEIGHT 16
unsigned char aFire[FIRE_WIDTH][FIRE_HEIGHT];

#define FIRE_COLOR_RANGE 60 // 6 colors

#define FIRE_X_ORIGIN 80 - (FIRE_WIDTH / 2)
#define FIRE_Y_ORIGIN 200

////////////////////////////////////////////////////////////////////////
//main
////////////////////////////////////////////////////////////////////////
void main()
{
	unsigned int nFPS = 0;
	unsigned int nTimeLast = 0;
	unsigned char nX = 0;
	unsigned char nY = 0;
	
	SetMode(0);
	SetBorder(0);
	SetPalette(FirePalette, NUM_COLORS);
  
  memset(aFire, 0, sizeof(aFire));
  
	nTimeLast = GetTime();

	while(1)
	{
		nFPS++;

		if(GetTime() - nTimeLast >= 300)
		{
			SetCursor(1, 1);
			printf("%u  ", nFPS);

			nTimeLast = GetTime();
			nFPS = 0;
		}
		
		//fills the botton line with random values
		for(nX = 1; nX < (FIRE_WIDTH - 1); nX++)
		 aFire[nX][0] = rand() % FIRE_COLOR_RANGE;
		 
		//interpolate colors from bottom to top
		for(nY = 1; nY < FIRE_HEIGHT; nY++)
		{
			for(nX = 0; nX < FIRE_WIDTH; nX++)
			{
		 		aFire[nX][nY] = (aFire[nX][nY] + aFire[nX][nY - 1] +
		 										 (nX > 0 ? aFire[nX - 1][nY - 1] : 0) +
		 										 (nX < FIRE_WIDTH - 1 ? aFire[nX + 1][nY - 1] : 0) -
		 										 (FIRE_COLOR_RANGE / FIRE_HEIGHT)) / 4;
		 	}
		}
		
		//screen
		for(nY = 1; nY < FIRE_HEIGHT; nY++)
		{
			for(nX = 0; nX < FIRE_WIDTH; nX++)
			{
				PutPixelMode0(FIRE_X_ORIGIN + nX, FIRE_Y_ORIGIN - nY, aFire[nX][nY] / 10 + (aFire[nX][nY] % 10 >= 5 ? 1 : 0));
		 	}
		}

	}
}
////////////////////////////////////////////////////////////////////////