////////////////////////////////////////////////////////////////////////
// Fire02.c
// Mochilote - www.cpcmania.com
////////////////////////////////////////////////////////////////////////
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


////////////////////////////////////////////////////////////////////////
//------------------------ Generic functions -------------------------//
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//GetTime()
////////////////////////////////////////////////////////////////////////
unsigned char char3,char4;

unsigned int GetTime()
{
	unsigned int nTime = 0;

	__asm
		CALL #0xBD0D ;KL TIME PLEASE
		PUSH HL
		POP DE
		LD HL, #_char3
		LD (HL), D
		LD HL, #_char4
		LD (HL), E
	__endasm;

	nTime = (char3 << 8) + char4;

	return nTime;
}
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//SetColor
////////////////////////////////////////////////////////////////////////
void SetColor(unsigned char nColorIndex, unsigned char nPaletteIndex)
{
	__asm
		ld a, 4 (ix)
		ld b, 5 (ix)
		ld c, b
		call #0xBC32 ;SCR SET INK
	__endasm;
}
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//SetPalette
////////////////////////////////////////////////////////////////////////
void SetPalette(const unsigned char *pPalette, unsigned char nNumColors)
{
  unsigned char nColor = 0;

  for(nColor = 0; nColor < nNumColors; nColor++)
    SetColor(nColor, pPalette[nColor]);
}
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//SetMode
////////////////////////////////////////////////////////////////////////
void SetMode(unsigned char nMode)
{
	__asm
		ld a, 4 (ix)
		call #0xBC0E ;SCR_SET_MODE
	__endasm;
}
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//SetBorder
////////////////////////////////////////////////////////////////////////
void SetBorder(unsigned char nColor)
{
	__asm
		ld b, 4 (ix)
		ld c, b
		call #0xBC38 ;SCR_SET_BORDER
	__endasm;
}
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//SetCursor
////////////////////////////////////////////////////////////////////////
void SetCursor(unsigned char nColum, unsigned char nLine)
{
	__asm
		ld h, 4 (ix)
		ld l, 5 (ix)
		call #0xBB75 ;TXT SET CURSOR
	__endasm;
}
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//---------------- Specific variables and functions-------------------//
////////////////////////////////////////////////////////////////////////

#define NUM_COLORS 6
const unsigned char FirePalette[NUM_COLORS] = {0, 3, 6, 15, 24, 26};

#define FIRE_WIDTH 32
#define FIRE_HEIGHT 16
unsigned char aFire[FIRE_WIDTH * FIRE_HEIGHT];

#define FIRE_COLOR_RANGE 60 // 6 colors
const unsigned char aFireColors[FIRE_COLOR_RANGE] = {
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0xC0, 0xC0, 0xC0, 0xC0, 0xC0, 0x0C, 0x0C, 0x0C, 0x0C, 0x0C,
		0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0x30, 0x30, 0x30, 0x30, 0x30,
		0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0,
		0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0};

#define FIRE_X_ORIGIN 40 - (FIRE_WIDTH / 2)
#define FIRE_Y_ORIGIN 199

unsigned int aScreenAddress[FIRE_HEIGHT * 2];

#define RANDOM_VALUES 1024
unsigned char aRandom[RANDOM_VALUES];

////////////////////////////////////////////////////////////////////////
//main
////////////////////////////////////////////////////////////////////////
void main()
{
	unsigned int nFPS = 0;
	unsigned int nTimeLast = 0;
	unsigned char nX = 0;
	unsigned char nY = 0;
	unsigned char *pFire1 = NULL;
	unsigned char *pFire2 = NULL;
	unsigned int *pScreenAddress = NULL;
	unsigned char *pScreen1 = NULL;
	unsigned char *pScreen2 = NULL;
	unsigned int nRandom = 0;
	unsigned char *pArray = NULL;
	
	SetMode(0);
	SetBorder(0);
	SetPalette(FirePalette, NUM_COLORS);
  
  memset(aFire, 0, sizeof(aFire));
  
  for(nY = 0; nY < FIRE_HEIGHT * 2; nY++)
  	aScreenAddress[nY] = 0xC000 + (((FIRE_Y_ORIGIN - nY) / 8) * 80) + (((FIRE_Y_ORIGIN - nY) % 8) * 2048) + FIRE_X_ORIGIN;
  
  for(nRandom = 0; nRandom < RANDOM_VALUES; nRandom++)
  	aRandom[nRandom] = FIRE_COLOR_RANGE / 4 + rand() % (FIRE_COLOR_RANGE * 3 / 4);
  
	nTimeLast = GetTime();

	while(1)
	{
		nFPS++;

		if(GetTime() - nTimeLast >= 300)
		{
			SetCursor(1, 1);
			printf("%u  ", nFPS);

			nTimeLast = GetTime();
			nFPS = 0;
		}
		
		//fills the botton line with random values
		pArray = (unsigned char *)aFire;
		
		for(nX = 1; nX < (FIRE_WIDTH - 1); nX++)
		{
		 *pArray++ = aRandom[nRandom++];

		 if(nRandom >= RANDOM_VALUES)
		 	nRandom = 0;
		}

		//interpolate colors from bottom to top
		pFire1 = aFire + FIRE_WIDTH; //current row
		pFire2 = aFire; //previous row
		 
		for(nY = 1; nY < FIRE_HEIGHT; nY++)
		{
			*pFire1 = (*pFire1 + *pFire2 + *(pFire2 + 1) -
									(FIRE_COLOR_RANGE / FIRE_HEIGHT)) / 4;
				
			pFire1++;
			pFire2++;
				
			for(nX = 1; nX < FIRE_WIDTH - 1; nX++)
			{
				
				*pFire1 = (*pFire1 + *pFire2 + *(pFire2 - 1) + *(pFire2 + 1) -
									(FIRE_COLOR_RANGE / FIRE_HEIGHT)) / 4;
				
				pFire1++;
				pFire2++;
		 	}

			*pFire1 = (*pFire1 + *pFire2 + *(pFire2 - 1) -
									(FIRE_COLOR_RANGE / FIRE_HEIGHT)) / 4;
				
			pFire1++;
			pFire2++;
		}
	
		//screen
		pFire1 = aFire + FIRE_WIDTH;
		pScreenAddress = aScreenAddress;
		for(nY = 1; nY < FIRE_HEIGHT; nY++)
		{
			pScreen1 = (unsigned char *)*pScreenAddress;
			pScreen2 = (unsigned char *)*(pScreenAddress + 1);
			pScreenAddress += 2;

			for(nX = 0; nX < FIRE_WIDTH; nX++)
			{
				*pScreen1++ = *pScreen2++ = aFireColors[*pFire1++];
		 	}
		}

	}
}
////////////////////////////////////////////////////////////////////////