////////////////////////////////////////////////////////////////////////
// Fire05.c
// Mochilote - www.cpcmania.com
////////////////////////////////////////////////////////////////////////
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


////////////////////////////////////////////////////////////////////////
//------------------------ Generic functions -------------------------//
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//GetTime()
////////////////////////////////////////////////////////////////////////
unsigned char char3,char4;

unsigned int GetTime()
{
	unsigned int nTime = 0;

	__asm
		CALL #0xBD0D ;KL TIME PLEASE
		PUSH HL
		POP DE
		LD HL, #_char3
		LD (HL), D
		LD HL, #_char4
		LD (HL), E
	__endasm;

	nTime = (char3 << 8) + char4;

	return nTime;
}
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//SetColor
////////////////////////////////////////////////////////////////////////
void SetColor(unsigned char nColorIndex, unsigned char nPaletteIndex)
{
	__asm
		ld a, 4 (ix)
		ld b, 5 (ix)
		ld c, b
		call #0xBC32 ;SCR SET INK
	__endasm;
}
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//SetPalette
////////////////////////////////////////////////////////////////////////
void SetPalette(const unsigned char *pPalette, unsigned char nNumColors)
{
  unsigned char nColor = 0;

  for(nColor = 0; nColor < nNumColors; nColor++)
    SetColor(nColor, pPalette[nColor]);
}
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//SetMode
////////////////////////////////////////////////////////////////////////
void SetMode(unsigned char nMode)
{
	__asm
		ld a, 4 (ix)
		call #0xBC0E ;SCR_SET_MODE
	__endasm;
}
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//SetBorder
////////////////////////////////////////////////////////////////////////
void SetBorder(unsigned char nColor)
{
	__asm
		ld b, 4 (ix)
		ld c, b
		call #0xBC38 ;SCR_SET_BORDER
	__endasm;
}
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//SetCursor
////////////////////////////////////////////////////////////////////////
void SetCursor(unsigned char nColum, unsigned char nLine)
{
	__asm
		ld h, 4 (ix)
		ld l, 5 (ix)
		call #0xBB75 ;TXT SET CURSOR
	__endasm;
}
////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////
//---------------- Specific variables and functions-------------------//
////////////////////////////////////////////////////////////////////////

#define NUM_COLORS 6
const unsigned char FirePalette[NUM_COLORS] = {0, 3, 6, 15, 24, 26};

#define FIRE_WIDTH 32
#define FIRE_HEIGHT 16

#define FIRE_COLOR_RANGE 60 // 6 colors
const unsigned char aFireColors[FIRE_COLOR_RANGE] = {
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0xC0, 0xC0, 0xC0, 0xC0, 0xC0, 0x0C, 0x0C, 0x0C, 0x0C, 0x0C,
		0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0x30, 0x30, 0x30, 0x30, 0x30,
		0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0,
		0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0};

#define FIRE_X_MARGIN 4
#define FIRE_X_ORIGIN 40 - (FIRE_WIDTH - FIRE_X_MARGIN)
#define FIRE_Y_ORIGIN 199

#define NUM_ADDRESS FIRE_HEIGHT * 4
unsigned int aScreenAddress[NUM_ADDRESS];

#define NUM_FRAMES 65
unsigned char aFireFrames[NUM_FRAMES][FIRE_WIDTH * FIRE_HEIGHT];

////////////////////////////////////////////////////////////////////////
//main
////////////////////////////////////////////////////////////////////////
void main()
{
	unsigned int nFPS = 0;
	unsigned int nTimeLast = 0;
	unsigned char nX = 0;
	unsigned char nY = 0;
	unsigned char nFrame = 0;
	unsigned char *pFire1 = NULL;
	unsigned char *pFire2 = NULL;
	unsigned int nPixel = 0;
	unsigned int *pScreenAddress = NULL;
	unsigned char *pScreen1 = NULL;
	unsigned char *pScreen2 = NULL;
	unsigned int nPreframes = 9;
	
	SetMode(0);
	SetBorder(0);
	SetPalette(FirePalette, NUM_COLORS);
  
  memset(aFireFrames, 0, sizeof(aFireFrames));
  
  for(nY = 0; nY < NUM_ADDRESS; nY++)
  	aScreenAddress[nY] = 0xC000 + (((FIRE_Y_ORIGIN - nY) / 8) * 80) + (((FIRE_Y_ORIGIN - nY) % 8) * 2048) + FIRE_X_ORIGIN;
  
  SetCursor(1, 1);
  printf("Calculating...");
  
  for(nFrame = 0; nFrame < NUM_FRAMES; nFrame++)
  {
  	if(nPreframes > 0)
  		nFrame = 0;
 
  	SetCursor(8, 5);
		printf("%u / %u  ", nFrame + 1, NUM_FRAMES);
		
		//fills the botton line with random values
		for(nX = 1; nX < (FIRE_WIDTH - 1); nX++)
		 aFireFrames[nFrame][nX] = FIRE_COLOR_RANGE / 4 + rand() % (FIRE_COLOR_RANGE * 3 / 4);

		//interpolate colors from bottom to top
		pFire1 = aFireFrames[nFrame] + FIRE_WIDTH; //current row
		pFire2 = aFireFrames[nFrame]; //previous row
		 
		for(nY = 1; nY < FIRE_HEIGHT; nY++)
		{
			*pFire1 = (*pFire1 + *pFire2 + *(pFire2 + 1) -
									(FIRE_COLOR_RANGE / FIRE_HEIGHT)) / 4;
				
			pFire1++;
			pFire2++;
				
			for(nX = 1; nX < FIRE_WIDTH - 1; nX++)
			{
				
				*pFire1 = (*pFire1 + *pFire2 + *(pFire2 - 1) + *(pFire2 + 1) -
									(FIRE_COLOR_RANGE / FIRE_HEIGHT)) / 4;
				
				pFire1++;
				pFire2++;
		 	}

			*pFire1 = (*pFire1 + *pFire2 + *(pFire2 - 1) -
									(FIRE_COLOR_RANGE / FIRE_HEIGHT)) / 4;
				
			pFire1++;
			pFire2++;
		}

  	if(nPreframes > 0)
			nPreframes--;
		
		if(nPreframes == 0)
		{
			if(nFrame < NUM_FRAMES - 1)
				memcpy(aFireFrames[nFrame + 1], aFireFrames[nFrame], FIRE_WIDTH * FIRE_HEIGHT);
	
			for(nPixel = 0; nPixel < FIRE_WIDTH * FIRE_HEIGHT; nPixel++)
				aFireFrames[nFrame][nPixel] = aFireColors[aFireFrames[nFrame][nPixel]];
		}
  }
  
  SetMode(0);
	nTimeLast = GetTime();
	nFrame = 0;

	while(1)
	{
		nFPS++;

		if(GetTime() - nTimeLast >= 300)
		{
			SetCursor(1, 1);
			printf("%u  ", nFPS);

			nTimeLast = GetTime();
			nFPS = 0;
		}
		
		//screen
		pFire1 = aFireFrames[nFrame++] + FIRE_WIDTH;

		if(nFrame >= NUM_FRAMES)
			nFrame = 0;

		pScreenAddress = aScreenAddress;
		for(nY = 1; nY < FIRE_HEIGHT; nY++)
		{
			for(nX = 0; nX < 4; nX++)
			{
				pScreen1 = (unsigned char *)*pScreenAddress++;
				memcpy(pScreen1, pFire1, FIRE_WIDTH - FIRE_X_MARGIN);
				memcpy(pScreen1 + FIRE_WIDTH - FIRE_X_MARGIN, pFire1 + FIRE_X_MARGIN, FIRE_WIDTH - FIRE_X_MARGIN);
			}

			pFire1 += FIRE_WIDTH;
		}
	}
}
////////////////////////////////////////////////////////////////////////